#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of calamares-settings steamos ===
#
#   SPDX-FileCopyrightText: 2014 Aurélien Gâteau <agateau@kde.org>
#   SPDX-FileCopyrightText: 2014 Anke Boersma <demm@kaosx.us>
#   SPDX-FileCopyrightText: 2014 Daniel Hillenbrand <codeworkx@bbqlinux.org>
#   SPDX-FileCopyrightText: 2014 Benjamin Vaudour <benjamin.vaudour@yahoo.fr>
#   SPDX-FileCopyrightText: 2014-2019 Kevin Kofler <kevin.kofler@chello.at>
#   SPDX-FileCopyrightText: 2015-2018 Philip Mueller <philm@manjaro.org>
#   SPDX-FileCopyrightText: 2016-2017 Teo Mrnjavac <teo@kde.org>
#   SPDX-FileCopyrightText: 2017 Alf Gaida <agaida@siduction.org>
#   SPDX-FileCopyrightText: 2017-2019 Adriaan de Groot <groot@kde.org>
#   SPDX-FileCopyrightText: 2017 Gabriel Craciunescu <crazy@frugalware.org>
#   SPDX-FileCopyrightText: 2017 Ben Green <Bezzy1999@hotmail.com>
#   SPDX-FileCopyrightText: 2019-2020 Collabora Ltd <arnaud.ferraris@collabora.com>
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   calamares-settings steamos is Free Software: see the License-Identifier above.
#

import os
import subprocess

import libcalamares

from libcalamares.utils import check_target_env_call


import gettext
_ = gettext.translation("calamares-python",
                        localedir=libcalamares.utils.gettext_path(),
                        languages=libcalamares.utils.gettext_languages(),
                        fallback=True).gettext

# This is the sanitizer used all over to tidy up filenames
# to make identifiers (or to clean up names to make filenames).
file_name_sanitizer = str.maketrans(" /()", "_-__")

def pretty_name():
    return _("Install SteamOS bootloader.")


def chroot_check_target_env_call(command):
    chroot = []
    if "chrootDirectory" in libcalamares.job.configuration:
        chroot = [ "chroot", libcalamares.job.configuration["chrootDirectory"]]

    return check_target_env_call(chroot + command)


def get_partuuid(mountpoint):
    """
    Checks and passes 'partuuid' to other routine.

    :param mountpoint:
    :return:
    """
    partitions = libcalamares.globalstorage.value("partitions")

    for partition in partitions:
        if partition["mountPoint"] == mountpoint:
            libcalamares.utils.debug("{!s} partuuid: \"{!s}\"".format(mountpoint, partition["partuuid"]))
            return partition["partuuid"]

    return ""


def get_devpath(device):
    devname = os.path.basename(device)
    return os.path.realpath(os.path.join("/sys/class/block", devname))


def efi_label():
    if "efiBootloaderId" in libcalamares.job.configuration:
        efi_bootloader_id = libcalamares.job.configuration[
                                "efiBootloaderId"]
    else:
        branding = libcalamares.globalstorage.value("branding")
        efi_bootloader_id = branding["bootloaderEntryName"]

    return efi_bootloader_id.translate(file_name_sanitizer)


def efi_boot_next(partition_uuid, bootloader_name):
    """
    Tell EFI to definitely boot into the just-installed
    system next time.

    :param partition_uuid:
    :param bootloader_name:
    """
    boot_mgr = libcalamares.job.configuration["efiBootMgr"]
    boot_entry = None
    efi_bootvars = subprocess.check_output([boot_mgr, "-v"], text=True)
    for line in efi_bootvars.split('\n'):
        if not line:
            continue
        words = line.split()
        if len(words) >= 3 and words[1] == bootloader_name and words[-1].find(partition_uuid.lower()) != -1:
            boot_entry = words[0][4:8]
            break
    if boot_entry:
        subprocess.call([boot_mgr, "-n", boot_entry])


def install_grub(efi_directory):
    """
    Installs grub as bootloader, either in pc or efi mode.

    :param efi_directory:
    """
    libcalamares.utils.debug("Bootloader: grub (efi)")
    install_path = libcalamares.globalstorage.value("rootMountPoint")
    install_efi_directory = install_path + efi_directory

    if not os.path.isdir(install_efi_directory):
        os.makedirs(install_efi_directory)

    efi_bootloader_id = efi_label()
    efi_target = "x86_64-efi"
    efi_grub_file = "grubx64.efi"
    efi_boot_file = "bootx64.efi"

    if "grubMkimage" in libcalamares.job.configuration:
        chroot_check_target_env_call([libcalamares.job.configuration["grubMkimage"],
                                      "--format=" + efi_target,
                                      "--output=" + os.path.join(efi_directory, "EFI",
                                                                 efi_bootloader_id,
                                                                 efi_grub_file)])
    else:
        chroot_check_target_env_call([libcalamares.job.configuration["grubInstall"],
                                      "--target=" + efi_target,
                                      "--efi-directory=" + efi_directory,
                                      "--bootloader-id=" + efi_bootloader_id,
                                      "--force"])

    # VFAT is weird, see issue CAL-385
    install_efi_directory_firmware = (vfat_correct_case(
                                          install_efi_directory,
                                          "EFI"))
    if not os.path.exists(install_efi_directory_firmware):
        os.makedirs(install_efi_directory_firmware)

    # there might be several values for the boot directory
    # most usual they are boot, Boot, BOOT

    install_efi_boot_directory = (vfat_correct_case(
                                      install_efi_directory_firmware,
                                      "boot"))
    if not os.path.exists(install_efi_boot_directory):
        os.makedirs(install_efi_boot_directory)

    # The input file /etc/default/grub should already be filled out by the
    # grubcfg job module.
    if "grubUpdate" in libcalamares.job.configuration:
        chroot_check_target_env_call([libcalamares.job.configuration["grubUpdate"]])
    else:
        chroot_check_target_env_call([libcalamares.job.configuration["grubMkconfig"],
                                      "-o", libcalamares.job.configuration["grubCfg"]])


def install_steamcl(esp_directory, efi_directory):
    """
    Installs steamcl as chainloader.

    :param esp_directory:
    """

    partitions = libcalamares.globalstorage.value("partitions")
    bootloader_name = efi_label()

    # Identify important devices
    efi_device = str()
    esp_device = str()
    for part in partitions:
        if part["mountPoint"] == efi_directory:
            efi_device = os.path.realpath(part["device"])
        if part["mountPoint"] == esp_directory:
            esp_device = os.path.realpath(part["device"])

    target_steamos_dir = vfat_correct_case(efi_directory, bootloader_name)
    if not os.path.exists(target_steamos_dir):
        os.makedirs(target_steamos_dir)

    # Workaround for some UEFI firmwares
    FALLBACK = "installEFIFallback"
    libcalamares.utils.debug("UEFI Fallback: " + str(libcalamares.job.configuration.get(FALLBACK, "<unset>")))
    extra_removable = libcalamares.job.configuration.get(FALLBACK, True)

    # ESP is on the same drive
    # Note: steamcl walks through media to find a bootloader to load, and has
    # its own algorithm to pick up the best candidate.
    # The flag restricted restricts the lookup to the same media.
    esp_parent_device = os.path.dirname(get_devpath(esp_device))
    efi_parent_device = os.path.dirname(get_devpath(efi_device))
    restricted = os.path.samefile(esp_parent_device, efi_parent_device)

    # Install chainloader
    if "steamclInstall" in libcalamares.job.configuration:
        libcalamares.utils.debug("Chainloader: steamcl")
        chroot_check_target_env_call([libcalamares.job.configuration["steamclInstall"]]
                                     + ([ "--force-extra-removable" ] if extra_removable else [])
                                     + ([ "--flags", "restricted" ] if restricted else []))

    # Create partition definitions
    if "steamosPartsets" in libcalamares.job.configuration:
        devices = []
        for part in partitions:
            # steamos partitions are the ones with a mountpoint
            if not part["mountPoint"]:
                continue
            devices.append(os.path.realpath(part["device"]))
        chroot_check_target_env_call([libcalamares.job.configuration["steamosPartsets"],
                                      "--devices", " ".join(devices),
                                      os.path.join(efi_directory, bootloader_name, "partsets")])

    # Generate a default SteamOS bootconf file
    if "steamosBootconf" in libcalamares.job.configuration:
        if "steamclTitle" in libcalamares.job.configuration:
            root_path = libcalamares.globalstorage.value("rootMountPoint")
            # configs live on the ESP in SteamOS/conf
            esp_confdir = os.path.join(root_path + esp_directory, bootloader_name, "conf")
            if not os.path.isdir(esp_confdir):
                os.makedirs(esp_confdir)
            # config is named for the image, eg: /esp/SteamOS/conf/A.conf
            image_id = libcalamares.job.configuration["steamclTitle"];
            conf_path = os.path.join(esp_confdir, image_id + ".conf")
            os.mknod(conf_path)
            check_target_env_call([libcalamares.job.configuration["steamosBootconf"],
                                          "--image", image_id,
                                          "config", "--set", "title", image_id])
    # Set the next boot entry
    if "efiBootMgr" in libcalamares.job.configuration:
        efi_boot_next(get_partuuid(esp_directory), efi_label())


def vfat_correct_case(parent, name):
    for candidate in os.listdir(parent):
        if name.lower() == candidate.lower():
            return os.path.join(parent, candidate)
    return os.path.join(parent, name)


def prepare_bootloader():
    """
    Prepares bootloader.

    :return:
    """
    esp_directory = libcalamares.globalstorage.value("efiSystemPartition")
    efi_directories = ["/efi"]

    for efi_directory in efi_directories:
        install_steamcl(esp_directory, efi_directory)
        install_grub(efi_directory)

def run():
    """
    Starts procedure and passes 'fw_type' to other routine.

    :return:
    """

    fw_type = libcalamares.globalstorage.value("firmwareType")

    if fw_type != "efi":
        libcalamares.utils.warning( "Non-EFI system." )
        return None

    partitions = libcalamares.globalstorage.value("partitions")
    efi_system_partition = libcalamares.globalstorage.value("efiSystemPartition")
    esp_found = [ p for p in partitions if p["mountPoint"] == efi_system_partition ]
    if not esp_found:
        libcalamares.utils.warning( "EFI system, but nothing mounted on {!s}".format(efi_system_partition) )
        return None

    prepare_bootloader()

    return None
