#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of calamares-settings steamos ===
#
#   SPDX-FileCopyrightText: 2020 Collabora Ltd
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   calamares-settings steamos is Free Software: see the License-Identifier above.
#

import subprocess

import libcalamares

from libcalamares.utils import check_target_env_call


import gettext
_ = gettext.translation("calamares-python",
                        localedir=libcalamares.utils.gettext_path(),
                        languages=libcalamares.utils.gettext_languages(),
                        fallback=True).gettext


def get_device_by_label(label):
    for entry in libcalamares.globalstorage.value("partitions"):
        if entry["name"] == label:
            return entry["device"]

    return None


def get_device_by_uuid(uuid):
    for entry in libcalamares.globalstorage.value("partitions"):
        if entry["uuid"] == uuid:
            return entry["device"]

    return None


def get_device_by_partlabel(partlabel):
    for entry in libcalamares.globalstorage.value("partitions"):
        if entry["partlabel"] == partlabel:
            return entry["device"]

    return None


def get_device_by_partuuid(partuuid):
    for entry in libcalamares.globalstorage.value("partitions"):
        if entry["partuuid"] == partuuid:
            return entry["device"]

    return None


def get_device(device):
    if device.startswith("LABEL="):
        return get_device_by_label(device.replace("LABEL=", ""))
    elif device.startswith("UUID="):
        return get_device_by_uuid(device.replace("UUID=", ""))
    elif device.startswith("PARTLABEL="):
        return get_device_by_partlabel(device.replace("PARTLABEL=", ""))
    elif device.startswith("PARTUUID="):
        return get_device_by_partuuid(device.replace("PARTUUID=", ""))
    else:
        return device


def pretty_name():
    return _("Tune filesystem parameters.")


def run():
    """
    Adjust tunable filesystem parameters on ext2/ext3/ext4 filesystems.

    :return:
    """

    if not "device" in libcalamares.job.configuration:
        return (_("Configuration Error"),
                _("No <pre>device</pre> defined."))

    device = libcalamares.job.configuration["device"]
    if device == "":
        return (_("Configuration Error"),
                _("<pre>device</pre> is empty."))

    device = get_device(device)
    if not device:
        return (_("Configuration Error"),
                _("No <pre>={}</pre> device not found.").format(device))

    # This doesn't work for btrfs but need to enable if we use ext4 rootfs
    '''
    cmd = ["tune2fs"]
    if "features" in libcalamares.job.configuration:
        features = libcalamares.job.configuration["features"].split()
        if features == "":
            return (_("Configuration Error"),
                    _("<pre>features</pre> is empty."))
        cmd += ["-O", ",".join(features)]

    if "uuid" in libcalamares.job.configuration:
        uuid = libcalamares.job.configuration["uuid"]
        if uuid == "":
            uuid = "clear"
        cmd += ["-U", uuid]
        subprocess.check_output(["e2fsck", "-y", "-f", device], text=True)

    cmd += [device]
    subprocess.check_output(cmd, text=True)
    '''
    # Btrfs : manually set readonly mode
    root_path = libcalamares.globalstorage.value("rootMountPoint");
    subprocess.check_output(["btrfs", "property", "set",
                             root_path, "ro", "true"], text=True)

    # Remove the 5% reserved blocks on /home, as requested by Valve.
    subprocess.check_output(["tune2fs", "-m", "0", get_device("PARTLABEL=home")], text=True)

    return None
